/*
 * file CcProvider.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * com.ibm.rational.wvcm.stp.cc.CcProvider
 *
 * (C) Copyright IBM Corporation 2004, 2012.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package com.ibm.rational.wvcm.stp.cc;

import java.util.List;

import javax.wvcm.PropertyRequestItem.PropertyRequest;
import javax.wvcm.ProviderFactory;
import javax.wvcm.ProviderFactory.Callback.Authentication;
import javax.wvcm.ResourceList;
import javax.wvcm.WorkspaceProvider;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.cc.CcMergeHandlingCallback.CheckinMergeHandling;
import com.ibm.rational.wvcm.stp.cc.CcView.TextMode;

/**
 * CcProvider is a ClearCase-specific extension of the generic
 * javax.wvcm.Provider interface and the Rational-specific
 * com.ibm.rational.wvcm.stp.StpProvider interface.
 * <p>
 * A CcProvider instance represents an individual user session with
 * ClearCase on a particular remote CCRC WAN server or with ClearCase on the
 * local machine.
 * </p>
 * <p>
 * In addition to the methods defined by its superinterfaces, the CcProvider
 * interface defines:
 * <bl> 
 * <li>Factory methods for constructing proxies
 * for various ClearCase-specific resources. Note that there are many resource
 * types where a ClearCase-specific interface exists for a standard WVCM
 * interface.  For instance, CcFile is the ClearCase specialization of
 * javax.wvcm.ControllableResource.
 * </li>
 * <li>Methods to register callbacks to notify your application of various
 * ClearCase-related events - "file area lock" exceptions, HTTP SSL certificate
 * exceptions, clearprompt events, etc.
 * </li>
 * <li>
 * Methods to get information about the ClearCase environment - the default
 * ClearCase registry region, whether MVFS is installed on the local machine, etc.
 * </li>
 * <li>
 * Methods for listing locally available ClearCase resources, e.g., 
 * {@link #getClientViewList}.
 * </bl>
 * </p>
 * <p>
 * NOTE: Starting in the 8.0 release, CM API providers can no
 * longer support both CC and CQ operations in the same provider instance.
 * In 7.1.x releases, a unified CM Server processed both CC and CQ operations.
 * In 8.0, there are separate CC and CQ servers, and so you must instantiate
 * a CqProvider to perform CQ operations and a CcProvider for CC operations.
 * </p>
 */
public interface CcProvider 
    extends StpProvider, WorkspaceProvider
{
    /**
     * <p>The name of a CcProvider class whose instances support ClearCase access
     * via an HTTP connection to a remote CCRC WAN server.  Use this
     * "network provider" to work with ClearCase web views hosted on a
     * particular server.
     * </p>
     * <p>Pass this name to <code>ProviderFactory.createProvider()</code> and cast
     * the resulting <code>Provider</code> instance to <code>CcProvider</code>
     * to access ClearCase-specific methods:
     * <pre>
     * CcProvider provider =
     *     (CcProvider) ProviderFactory.createProvider(
     *         CcProvider.NETWORK_PROVIDER_CLASS, ...);
     * </pre>
     * </p>
     * @see ProviderFactory#createProvider(String, javax.wvcm.ProviderFactory.Callback)
     */
    String NETWORK_PROVIDER_CLASS =
        "com.ibm.rational.stp.client.internal.cc.CcNetworkProviderImpl";

    /**
     * <p>The name of a CcProvider class whose instances support direct acess to
     * ClearCase installed on the local machine.  Use this "local provider" to
     * work with ClearCase dynamic views on the local machine.
     * </p>
     * <p>Pass this name to <code>ProviderFactory.createProvider()</code> and cast
     * the resulting <code>Provider</code> instance to <code>CcProvider</code>
     * to access ClearCase-specific methods:
     * <pre>
     * CcProvider provider =
     *     (CcProvider) ProviderFactory.createProvider(
     *         CcProvider.LOCAL_PROVIDER_CLASS, ...);
     * </pre>
     * </p>
     * @see ProviderFactory#createProvider(String, javax.wvcm.ProviderFactory.Callback)
     */
    String LOCAL_PROVIDER_CLASS =
        "com.ibm.rational.stp.client.internal.cc.CcLocalProviderImpl";

    /**
     * An extension of ProviderFactory.Callback that client call-backs can
     * return to provide a Primary Group or Group List for a ClearCase login
     */
    public interface CcAuthentication extends Authentication {

        /**
         * <p>ClearCase checks the user's "primary group" to determine whether
         * the user can peform certain operations in a given VOB, such as
         * creating a new CC element.
         * </p>
         * <p>On Windows operating systems, the user's primary group cannot be
         * reliably determined, so it must be set explicitly here.
         * </p>
         * <p>On Unix, this setting may be used to override the user's primary
         * group as specified in the /etc/password file or equivalent.
         * </p>
         * @return The primary group name to use performing ClearCase operations,
         *         or <code>null</code> to use the default primary group
         */
        String getPrimaryGroupName();

        /**
         * <p>ClearCase checks the user's group list (the list of OS groups
         * to which the user belongs) to determine whether the user can peform
         * certain operations in a given VOB.
         * </p>
         * <p>If the user belongs to more than 32 groups, in certain situations
         * ClearCase may ignore some of those groups, causing the operation to
         * fail unnecessarily.  In this case, use this setting to define which
         * groups (up to 32) ClearCase should use.
         * </p>
         * @return The group list to use when performing ClearCase operations,
         *         or empty list to use the default group list
         */
        List<String> getGroupList();
    }

    /**
     * Is this a local provider, as opposed to a network provider?
     * @see CcProvider#LOCAL_PROVIDER_CLASS
     * @see CcProvider#NETWORK_PROVIDER_CLASS
     */
    public boolean isLocalProvider();
    
    /**
     * Is ClearCase with MVFS version 8.0.0.0 or greater installed on this client?
     */
    public boolean isMVFSInstalledLocally();
    
    /**
     * Get the server's default ClearCase registry region.
     * @param wantedProps list of properties to be returned with registry region proxy
     * @return proxy for server's default registry region.
     * @throws WvcmException
     */
    public CcRegistryRegion doGetDefaultCcRegistryRegion(PropertyRequest wantedProps) throws WvcmException;

    /**
     * @return server's version, as a string.
     * @throws WvcmException
     */
    public String doGetServerVersionString() throws WvcmException;

    /**
     * <p>
     * Get the default text mode for web views. This depends on two pieces
     * of information: the CCRC WAN server's default VOB line termination
     * setting and the client OS. It is defined as follows: 
     * </p>
     * 
     * <table border="1">
     * <tr>
     * <th>server default VOB line term</th>
     * <th>client OS</th>
     * <th>web view default text mode</th>
     * </tr>
     * <td>LF (UNIX)</td>
     * <td>UNIX</td>
     * <td>{@link TextMode#TRANSPARENT}</td>
     * </tr>
     * <tr>
     * <td>LF (UNIX)</td>
     * <td>Windows</td>
     * <td>{@link TextMode#INSERT_CR}</td>
     * </tr>
     * <tr>
     * <td>CR-LF (MSDOS)</td>
     * <td>UNIX</td>
     * <td>{@link TextMode#STRIP_CR}</td>
     * </tr>
     * <tr>
     * <td>CR-LF (MSDOS)</td>
     * <td>Windows</td>
     * <td>{@link TextMode#TRANSPARENT}</td>
     * </tr>
     * <tr>
     * <td>No default set</td>
     * <td>Both UNIX and Windows</td>
     * <td>{@link TextMode#TRANSPARENT}</td>
     * </tr>
     * </table>
     * 
     * @return enumeration representing the view's default text mode
     * @throws WvcmException
     */
    public TextMode doGetDefaultViewTextMode() throws WvcmException;

    /**
     * Get the list of views that are accessible on the local machine.
     * This includes all web views listed in the local web view registry
     * (typically ".ccase_wvreg" in the user's home directory).
     * In addition, if this provider is a local provider (LOCAL_PROVIDER_CLASS),
     * the list includes any dynamic views currently running on this machine.
     * <p>
     * If the caller has registered a ProviderMangerCallback on this provider,
     * that callback will be invoked for each view.  Because these views may be
     * hosted on different CCRC WAN servers, this gives the caller an opportunity to
     * specify which provider should be used for each view.
     * </p>
     * @param wantedProps list of properties to retrieve for each view proxy.
     * @return list of locally accessible views as a list of CcView proxies
     * @see CcProviderManagerCallback
     * @see CcProvider#registerProviderManagerCallback(CcProviderManagerCallback)
     */
    public ResourceList<CcView> getClientViewList(PropertyRequest wantedProps)
        throws WvcmException;

    /**
     * Create a proxy for a ClearCase UCM activity.
     * @param   location  Location for UCM activity.
     * @return  UCM activity proxy.
     */
    public CcActivity ccActivity(StpLocation location);

    /**
     * Create a proxy for a ClearCase branch.
     * @param   location  Location for branch.
     * @return  branch proxy.
     */
    public CcBranch ccBranch(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM baseline.
     * @param   location  Location for UCM baseline.
     * @return  UCM baseline proxy.
     */
    public CcBaseline ccBaseline(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM component.
     * @param   location  Location for UCM component.
     * @return  UCM component proxy.
     */
    public CcComponent ccComponent(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM project.
     * @param   location  Location for UCM project.
     * @return  UCM project proxy.
     */
    public CcProject ccProject(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM project folder.
     * @param   location  Location for UCM project folder.
     * @return  UCM project folder proxy.
     */
    public CcProjectFolder ccProjectFolder(StpLocation location);

    /**
     * Create a proxy for a ClearCase UCM stream.
     * @param   location  Location for UCM stream.
     * @return  UCM stream proxy.
     */
    public CcStream ccStream(StpLocation location);

    /**
     * Construct a proxy for the CcDirectory (directory in a ClearCase view)
     * specified by the given location.
     * @param loc the location of the directory
     * @return a new CcDirectory proxy for a directory at this Location.
     */
    public CcDirectory ccDirectory(StpLocation loc);

    /**
     * Construct a proxy for the CcFile (file in a ClearCase view)
     * specified by the given location.
     * @param loc the location of the file.
     * @return a new CcFile proxy for a file at this Location.
     */
    public CcFile ccFile(StpLocation loc);

    /**
     * Construct a directory version proxy for the given location.
     * @param loc the location of the directory version.
     * @return a new proxy for a directory version at this Location.
     */
    public CcDirectoryVersion ccDirectoryVersion(StpLocation loc);

    /**
     * Construct a version proxy for the given location.
     * @param loc the location of the version.
     * @return a new proxy for a version at this Location.
     */
    public CcVersion ccVersion(StpLocation loc);

    /**
     * Construct a element proxy for the given location.
     * @param loc the location of the element.
     * @return a new proxy for a element at this Location.
     */
    public CcElement ccElement(StpLocation loc);

    /**
     * Construct a registry region proxy for the given location.
     * @param loc the location of the registry region
     * @return a new proxy for the registry region at this location
     */
    public CcRegistryRegion ccRegistryRegion(StpLocation loc);
    
    /**
     * Construct a VOB proxy for the given location.
     * @param loc the location of the VOB.
     * @return a new proxy for a VOB at this Location.
     */
    public CcVob ccVob(StpLocation loc);

    /**
     * Construct a VOB tag proxy for the given location.
     * @param loc the location of the VOB tag.
     * @return a new proxy for a VOB tag at this Location.
     */
    public CcVobTag ccVobTag(StpLocation loc);

    /**
     * Construct a view proxy for the given location.
     * @param loc the location of the view.
     * @return a new proxy for a view at this Location.
     */
    public CcView ccView(StpLocation loc);

    /**
     * Construct a view tag proxy for the given location.
     * @param loc the location of the view tag.
     * @return a new proxy for a view tag at this Location.
     */
    public CcViewTag ccViewTag(StpLocation loc);

    /**
     * Construct a storage location proxy for the given location.
     * @param loc the location of the storage location.
     * @return a new proxy for a storage location at this Location.
     */
    public CcStorageLocation ccStorageLocation(StpLocation loc);

    /**
     * Construct a attribute type proxy for the given location.
     * @param loc the location of the attribute type.
     * @return a new proxy for a attribute type at this Location.
     */
    public CcAttributeType ccAttributeType(StpLocation loc);

    /**
     * Construct a branch type proxy for the given location.
     * @param loc the location of the branch type.
     * @return a new proxy for a branch type at this Location.
     */
    public CcBranchType ccBranchType(StpLocation loc);

    /**
     * Construct a element type proxy for the given location.
     * @param loc the location of the element type.
     * @return a new proxy for a element type at this Location.
     */
    public CcElementType ccElementType(StpLocation loc);

    /**
     * Construct a hyperlink type proxy for the given location.
     * @param loc the location of the hyperlink type.
     * @return a new proxy for a hyperlink type at this Location.
     */
    public CcHyperlinkType ccHyperlinkType(StpLocation loc);

    /**
     * Construct a label type proxy for the given location.
     * @param loc the location of the label type.
     * @return a new proxy for a label type at this Location.
     */
    public CcLabelType ccLabelType(StpLocation loc);

    /**
     * Construct an empty CcLockInfo object for using
     * as a property to set on a CcVobResource.
     * @return a new lock information object
     */
    public CcLockInfo CcLockInfo();
   
    /**
     * Construct a replica proxy for the given location.
     * @param loc the location of the replica.
     * @return a new proxy for a replica at this Location.
     */
    public CcReplica ccReplica(StpLocation loc);

    /**
     * Construct a symbolic link proxy for the given location.
     * @param loc the location of the symbolic link.
     * @return a new proxy for a symbolic link at this Location.
     */
    public CcSymlink ccSymlink(StpLocation loc);

    /**
     * Construct a reference to a task.
     * @param uri Full URI of the task.
     * @return a new task object.
     */
    @Deprecated
    public CcTask ccTask(String uri);
    
    /**
     * Construct a reference to a task.
     * @param loc the location of the task
     * @return a new task object.
     */
    public CcTask ccTask(StpLocation loc);
    
    /**
     * Construct a trigger type proxy for the given location.
     * @param loc the location of the trigger type.
     * @return a new proxy for a trigger type at this Location.
     */
    public CcTriggerType ccTriggerType(StpLocation loc);
    
    /**
     * Register a provider manager callback.
     * <p>
     * Various CM API operations invoke this callback in situations where a new
     * provider may be required to complete an operation.  For instance, if you
     * have local web views hosted on different CCRC WAN servers, the
     * {@link #getClientViewList} method will invoke this callback for each view
     * to allow your application to associate the correct provider for that view.
     * </p>
     * <p>
     * Various ClearQuest/UCM Integration operations may also invoke this
     * callback to obtain a ClearQuest provider (CqProvider) instance.
     * </p>
     * <p>
     * The CM API uses a default implementation of this callback if the caller
     * does not register another one.
     * </p>
     * <p>
     * This callback is shared among all CcProvider instances, so it only needs
     * to be set once.
     * </p>
     * @param callback callback
     * @return the previously registered provider manager callback
     * @see CcProvider#getClientViewList(javax.wvcm.PropertyRequestItem.PropertyRequest)
     */
    public CcProviderManagerCallback
    registerProviderManagerCallback(CcProviderManagerCallback callback);
    
    /**
     * Register a callback to process ClearPrompt interaction requests.
     * @param callback callback
     */
    public void
    registerClearPromptCallback(CcClearPromptCallback callback);

    /**
     * Register a callback to handle SSL certificate conflicts.
     * The callback should be registered before attempting to authenticate over
     * a secure (SSL) HTTP connection in order to correctly handle certificate
     * exceptions.
     * <p>
     * This callback is shared among all CcProvider instances, so it only needs
     * to be set once.
     * </p>
     * @param callback Trust Manager callback to handle certificate
     * @throws WvcmException 
     */
    public void
    registerTrustManagerCallback(CcTrustManagerCallback callback) 
        throws WvcmException;
    
    /**
     * Register a callback to handle a FileAreaLockedException.
     * @param callback The new callback
     * @return The previous callback
     */
    public CcFileAreaLockedCallback registerCcFileAreaLockedCallback(
            CcFileAreaLockedCallback callback) throws WvcmException;
    
    /**
     * Register a callback to handling manual merges.
     * @param callback The new callback
     * @return The previous callback
     */
    public CcMergeHandlingCallback registerMergeHandlingCallback(
            CcMergeHandlingCallback callback) throws WvcmException;
    
    /**
     * Register a callback to handle state transitions of ClearQuest records. 
     * The callback is invoked if the transition involves required 
     * fields that the user needs to provide.
     * @param callback the callback we're registering
     * @return the previous registered callback
     */
    public CqRecordAutoTransitionCallback registerCqRecordAutoTransitionCallback(
            CqRecordAutoTransitionCallback callback) throws WvcmException;
    
    /**
     * Handle the manual merge using the provider's callback.
     * @return CheckinMergeHandling specifying how the checkin 
     *         is to be handled after the merge
     */
    public CheckinMergeHandling handleManualMerge(
            CcFile file,
            CcVersion fromVersion,
            CcVersion toVersion) throws WvcmException;

    /**
     * Performs a server logout and invalidates this provider's current
     * session.  Attempting to reuse this provider will result
     * in an attempt to create a new session using the previously 
     * registered authentication callback.
     */
    public void doLogout() throws WvcmException;
}
